#include <wx/wx.h>
#include <wx/filename.h>
#include "ArqWx.FileEditDialog.h"


namespace ArqWx
{

int idBtnOK = ::wxNewId();
int idBtnDefault = ::wxNewId();
int idBtnCancel = ::wxNewId();


BEGIN_EVENT_TABLE(FileEditDialog, wxDialog)
    EVT_BUTTON(idBtnOK, FileEditDialog::OnClickOK)
    EVT_BUTTON(idBtnDefault, FileEditDialog::OnClickDefault)

    EVT_CLOSE(FileEditDialog::OnCloseWindow)
END_EVENT_TABLE()


// FileEditDialog(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, const wxString& editedFilename, FileNameDisplayType displayedType)
//
// Constructor
//
// parameters:
// - parent (wxWindow*): parent window
// - id (wxWindowID): dialog identificator
// - title (const wxString&): dialog title
// - pos (const wxPoint&): dialog position
// - size (const wxSize&): dialog size
// - editedFilename (const wxString&): filename to edit
// - displayedType (FileNameDisplayType/FILENAME_DISPLAY_FULLNAME): type of display for file name (box label) (FILENAME_DISPLAY_NONE/FILENAME_DISPLAY_GENERIC/FILENAME_DISPLAY_FULLNAME/FILENAME_DISPLAY_FULLPATH)
// - fixedPitchFont (bool/true): text font style; use a fixed pitch font if true, otherwise use default font

FileEditDialog::FileEditDialog(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, const wxString& editedFilename, FileNameDisplayType displayedType, bool fixedPitchFont)
: wxDialog(parent, id, title, pos, size, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
, fileName(editedFilename)
{
    wxString displayedFileName;
    switch ( displayedType ) {
        case FILENAME_DISPLAY_NONE:
            displayedFileName = "";
            break;
        case FILENAME_DISPLAY_GENERIC:
            displayedFileName = "File content";
            break;
        case FILENAME_DISPLAY_FULLNAME:
            displayedFileName = wxFileName(this->fileName).GetFullName();
            break;
        case FILENAME_DISPLAY_FULLPATH:
            displayedFileName = this->fileName;
            break;
    }

    //* drawing of GUI
    wxPanel* panelMain = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
    wxBoxSizer* sizerMain = new wxBoxSizer(wxVERTICAL);
    panelMain->SetSizer(sizerMain);

        wxPanel* panelFileContent = new wxPanel(panelMain, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
        sizerMain->Add(panelFileContent, 1, wxEXPAND);
        wxStaticBoxSizer* sizerFileContent = new wxStaticBoxSizer(wxVERTICAL, panelFileContent, displayedFileName);
        panelFileContent->SetSizer(sizerFileContent);

            this->txtctrlFileContent = new wxTextCtrl(panelFileContent, -1, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxHSCROLL | wxTE_PROCESS_TAB | wxTE_RICH);
            sizerFileContent->Add(this->txtctrlFileContent, 1, wxEXPAND);

            if ( fixedPitchFont ) {
                wxTextAttr txtAttr = this->txtctrlFileContent->GetDefaultStyle();
                wxFont txtctrlFont = txtAttr.GetFont();
                txtctrlFont.SetFamily(wxFONTFAMILY_MODERN);
                txtAttr.SetFlags(wxTEXT_ATTR_FONT_FACE);
                txtAttr.SetFont(txtctrlFont);
                this->txtctrlFileContent->SetDefaultStyle(txtAttr);

                this->txtctrlFileContent->AppendText(" "); // necessary for default style to be updated when loading file
            }

        sizerMain->AddSpacer(5);

        wxPanel* panelCommands = new wxPanel(panelMain, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
        sizerMain->Add(panelCommands, 0, wxALIGN_CENTRE);
        wxBoxSizer* sizerCommands = new wxBoxSizer(wxHORIZONTAL);
        panelCommands->SetSizer(sizerCommands);

            sizerCommands->AddSpacer(10);

            this->btnOK = new wxButton(panelCommands, idBtnOK, "OK", wxDefaultPosition, wxDefaultSize);
            sizerCommands->Add(this->btnOK, 0, wxEXPAND);

            sizerCommands->AddSpacer(10);

            this->btnDefault = new wxButton(panelCommands, idBtnDefault, "Default", wxDefaultPosition, wxDefaultSize);
            sizerCommands->Add(this->btnDefault, 0, wxEXPAND);

            sizerCommands->AddSpacer(10);

            this->btnCancel = new wxButton(panelCommands, idBtnCancel, "Cancel", wxDefaultPosition, wxDefaultSize);
            sizerCommands->Add(this->btnCancel, 0, wxEXPAND);

        sizerMain->AddSpacer(5);

    this->btnOK->SetFocus(); // sets focus on 'OK' button and not on textctrl

    this->SetAffirmativeId(idBtnOK); // sets 'OK' button as affirmative button
    this->SetEscapeId(idBtnCancel); // sets 'Cancel' button as escape button

    this->SetClientSize(this->GetSize()); // forces the dialog to refresh

    //* initializations
    if ( !this->txtctrlFileContent->LoadFile(this->fileName) ) { // loads file and if it isn't successfull
        this->txtctrlFileContent->Clear(); // clears textcontrol
    }
}


//*** button events

void FileEditDialog::OnClickOK(wxCommandEvent& WXUNUSED(event))
{
    if ( this->txtctrlFileContent->IsModified() ) { // if file content has been modified
        this->txtctrlFileContent->SaveFile(this->fileName); // saves new content to file
        this->EndModal(wxID_OK); // ends the dialog with 'ok' as return code
    }
    else { // if file content has not been modified
        this->EndModal(wxID_CANCEL); // ends the dialog with 'cancel' as return code
    }

}

void FileEditDialog::OnClickDefault(wxCommandEvent& WXUNUSED(event))
{
    this->txtctrlFileContent->LoadFile(this->fileName); // reloads file
}

void FileEditDialog::OnCloseWindow(wxCloseEvent& WXUNUSED(event))
{
    this->EndModal(wxID_CANCEL); // ends the dialog with 'cancel' as return code
}

} // ArqWx namespace end
